--*****************************************************************************
--* File: lua/modules/ui/game/enhancement.lua
--* Author: Ted Snook
--* Summary: UI for the Enhancement Window
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Text = import('/lua/modules/maui/text.lua').Text
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Button = import('/lua/modules/maui/button.lua').Button

local dialog = false
local parent = false

local mfdControl = false
local gameParent = false

local controls = {
    dialog = false,
    hideButton = false,
    unitImage = false,
    slotButtons = {},
}

function CreateElements()
    controls.dialog = Bitmap(parent, UIUtil.UIFile('/dialogs/enhance-panel/panel_bmp.dds'))
    controls.hideButton = Button(controls.dialog, 
        UIUtil.UIFile('/dialogs/close_btn/close_btn_up.dds'),
        UIUtil.UIFile('/dialogs/close_btn/close_btn_over.dds'),
        UIUtil.UIFile('/dialogs/close_btn/close_btn_down.dds'),
        UIUtil.UIFile('/dialogs/close_btn/close_btn_dis.dds'))
    controls.unitImage = Bitmap(controls.dialog, UIUtil.UIFile('/dialogs/enhancements/default_unit.dds'))
    controls.windowTitle = UIUtil.CreateText(controls.dialog, "<LOC ENHANCE_0000>Title Bar", 14, UIUtil.bodyFont)
end

function DoBottomLayout()
    controls.dialog.Bottom:Set(function() return gameParent.Bottom() + 15 end)
    controls.dialog.Left:Set(function() return mfdControl.Right() - 31 end)
end

function DoLeftLayout()
    controls.dialog.Bottom:Set(function() return gameParent.Bottom() - 90 end)
    controls.dialog.Left:Set(mfdControl.Right)
end

function DoRightLayout()
    controls.dialog.Bottom:Set(function() return gameParent.Bottom() - 90 end)
    controls.dialog.Left:Set(gameParent.Left)
end

function DoMiniLayout()
    controls.dialog.Bottom:Set(function() return gameParent.Bottom() - 90 end)
    LayoutHelpers.AtHorizontalCenterIn(controls.dialog, gameParent, -25)
end

function CommonLayout()
    controls.dialog.Depth:Set(1000)
    
    controls.hideButton.Right:Set(function() return controls.dialog.Right() - 26 end)
    controls.hideButton.Top:Set(function() return controls.dialog.Top() + 15 end)
    
    LayoutHelpers.AtCenterIn(controls.unitImage, controls.dialog)
    #controls.unitImage.Top:Set(function() return controls.dialog.Top() + 40 end)
    controls.unitImage:DisableHitTest()
    
    controls.windowTitle.Top:Set(function() return controls.dialog.Top() + 18 end)
    controls.windowTitle.Left:Set(function() return controls.dialog.Left() + 35 end)
end

function SetLayout(layoutVar)
    if layoutVar == 'bottom' then
        DoBottomLayout()
    elseif layoutVar == 'left' then
        DoLeftLayout()
    elseif layoutVar == 'right' then
        DoRightLayout()
    elseif layoutVar == 'minimal' then
        DoMiniLayout()
    end
    CommonLayout()
end

function CreateDialog(layoutVar, inmfdControl, ingameParent, inMapGroup)
    parent = inMapGroup
    mfdControl = inmfdControl
    gameParent = ingameParent
    CreateElements()
    SetLayout(layoutVar)
            
    controls.hideButton.OnClick = function(self)
        controls.dialog:Hide()
        import('/lua/modules/ui/game/construction.lua').ToggleEnhancementWindow(false, controls.dialog)
        import('/lua/modules/ui/game/queue.lua').ToggleEnhancementWindow(false, controls.dialog)
    end
    controls.dialog:Hide() 
end

function ToggleDialog(enhancementTable, unitID, closeBool)
    if controls.dialog then 
        if closeBool or not controls.dialog:IsHidden() then
            if not controls.dialog:IsHidden() then
                import('/lua/modules/ui/game/construction.lua').ToggleEnhancementWindow(false, controls.dialog)
                import('/lua/modules/ui/game/queue.lua').ToggleEnhancementWindow(false, controls.dialog)
            end
            controls.dialog:Hide() 
        else
            local tempbp = unitID:GetBlueprint()
            controls.windowTitle:SetText(LOC(tempbp.Description))
            for i, v in controls.slotButtons do
                if controls.slotButtons[i] then
                    controls.slotButtons[i]:Destroy()
                    controls.slotButtons[i] = false
                end
            end
                
            controls.dialog:Show() 
            import('/lua/modules/ui/game/construction.lua').ToggleEnhancementWindow(true, controls.dialog)
            import('/lua/modules/ui/game/queue.lua').ToggleEnhancementWindow(true, controls.dialog)
            local blueprintID = unitID:GetBlueprint().BlueprintId
            local entityID = unitID:GetEntityId()
            local blueprint = unitID:GetBlueprint()
            
            controls.unitImage:SetTexture(UIUtil.UIFile('/dialogs/enhance-panel/'..blueprintID..'_bmp.dds'))
			            
            if enhancementTable.Slots then
                for i, v in enhancementTable.Slots do
                    if controls.slotButtons[i] then 
                        controls.slotButtons[i]:Destroy()
                        controls.slotButtons[i] = {}
                    else
                        controls.slotButtons[i] = {}
                    end
                    controls.slotButtons[i] = Checkbox(controls.unitImage, GetSlotTextures(blueprintID, i))
                    local y = v['y']
                    local x = v['x']
                    local slotName = i
                    controls.slotButtons[i].SlotName = slotName
                    controls.slotButtons[i].UnitID = unitID
                    controls.slotButtons[i].Blueprint = blueprint
                    controls.slotButtons[i].EntityID = entityID
                    controls.slotButtons[i].ButtonID = i
                    #controls.slotButtons[i].Top:Set(function() return controls.unitImage.Top() + y end)
                    #controls.slotButtons[i].Left:Set(function() return controls.unitImage.Left() + x end)
                    LayoutHelpers.AtCenterIn(controls.slotButtons[i], controls.dialog)
                    controls.slotButtons[i]:UseAlphaHitTest(true)
                    controls.slotButtons[i].OnClick = function(self, modifiers)
                        if not self:IsChecked() then
                            import('/lua/modules/ui/game/construction.lua').EnhancementPopulate(slotName)
                            for curIndex, curBtn in controls.slotButtons do
                                if self.ButtonID == curIndex then
                                    curBtn:SetCheck(true, true)
                                else
                                    curBtn:SetCheck(false, true)
                                end
                            end
                        end
                    end
                    controls.slotButtons[i].HandleEvent = function(self, event)
                        if event.Type == 'MouseEnter' then
                            local tempTable = DisplayText(blueprintID, self.Blueprint, self.SlotName, self.EntityID, self.UnitID)
                            if tempTable.Name != 'EMPTY' then
                                import('/lua/modules/ui/game/unitviewDetail.lua').ShowEnhancement(DisplayText(blueprintID, self.Blueprint, self.SlotName, self.EntityID, self.UnitID))
                            end
                        elseif event.Type == 'MouseExit' then
                            import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
                        end
                        Checkbox.HandleEvent(self, event)
                    end
                end
            else
                WARN("Enhancement Window: No slots defined in blueprint!")
            end
            
            if table.getsize(enhancementTable.Slots) == 1 then
                for i, v in controls.slotButtons do
                    if controls.slotButtons[i] then
                        controls.slotButtons[i]:Disable()
                    end
                end
                local tempSlotName = ""
                for i, v in enhancementTable.Slots do
                    tempSlotName = i
                end
                import('/lua/modules/ui/game/construction.lua').EnhancementPopulate(tempSlotName)
            else
                for i, v in controls.slotButtons do
                    if controls.slotButtons[i] then
                        controls.slotButtons[i]:SetCheck(true)
                        import('/lua/modules/ui/game/construction.lua').EnhancementPopulate(controls.slotButtons[i].ButtonID)
                        break
                    end
                end
            end
        end
    end
end
        
function DisplayText(unitID, blueprint, slot, entityID, userunit)
    local curEnhString = "EMPTY"
    local curEnhancement = ""
    local iconID = ""
    local enhTable = import('/lua/modules/enhancementcommon.lua').GetEnhancements(entityID)
    if enhTable then
        for i, v in enhTable do
            if slot == i then
                curEnhancement = v
            end
        end
        for i, v in blueprint.Enhancements do   
            if i == curEnhancement then
                if v.Name then
                    curEnhString = LOC(v.Name)
                    iconID = v.Icon
                end
            end
        end
    end
    return {Slot = LOC(slot), Name = curEnhString}, unitID, iconID, userunit
end
        
function GetSlotTextures(unitID, slotName)
    local tempBmpname = ""
    if slotName == "Back" then
        tempBmpName = "b"
    elseif slotName == "LCH" then
        tempBmpName = "la"
    elseif slotName == "RCH" then
        tempBmpName = "ra"
    elseif slotName == "Shield" then
        tempBmpName = "ch"
    else
    	tempBmpName = slotName
    
    end
   
    local iconName = UIUtil.UIFile('/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_up.dds')
    local upIconName = UIUtil.UIFile('/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_sel.dds')
    local downIconName = UIUtil.UIFile('/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_down.dds')
    local overIconName = UIUtil.UIFile('/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_over.dds')
   
   
   
    return iconName, upIconName, overIconName, upIconName, upIconName, downIconName, "UI_Enhancements_Click", "UI_Enhancements_Rollover"
end 

function DestroyDialog()
    if controls.dialog then
        controls.dialog:Hide()
    end
end

function Contract()
    controls.dialog:Hide()
end