--*****************************************************************************
--* File: lua/modules/ui/game/gamemain.lua
--* Author: Chris Blackwell
--* Summary: Entry point for the in game UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local WldUIProvider = import('/lua/modules/ui/game/wlduiprovider.lua').WldUIProvider
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Movie = import('/lua/modules/maui/movie.lua').Movie

local gameParent = false
local controlClusterGroup = false
local statusClusterGroup = false
local mapGroup = false
local mfdControl = false
local ordersControl = false
local queueControl = false

local isReplay = false

-- check this flag to see if it's valid to show the exit dialog
supressExitDialog = false

function GetReplayState()
    return isReplay
end

-- query this to see if the UI is hidden
gameUIHidden = false

PostScoreVideo = false

function SetLayout(layout)
    import('/lua/modules/ui/game/construction.lua').SetLayout(layout)
    import('/lua/modules/ui/game/borders.lua').SetLayout(layout)
    import('/lua/modules/ui/game/multifunction.lua').SetLayout(layout)
    if isReplay then
        import('/lua/modules/ui/game/replay.lua').SetLayout(layout)
    else
        import('/lua/modules/ui/game/orders.lua').SetLayout(layout)
    end
    import('/lua/modules/ui/game/unitview.lua').SetLayout(layout)
    import('/lua/modules/ui/game/objectives2.lua').SetLayout(layout)
    import('/lua/modules/ui/game/enhancements.lua').SetLayout(layout)
    import('/lua/modules/ui/game/selectionmanager.lua').SetLayout(layout)
    import('/lua/modules/ui/game/queue.lua').SetLayout(layout)
    import('/lua/modules/ui/game/unitviewDetail.lua').SetLayout(layout, mapGroup)
    import('/lua/modules/ui/game/economy.lua').SetLayout(layout)
    import('/lua/modules/ui/game/options.lua').SetLayout(layout)
    import('/lua/modules/ui/game/missiontext.lua').SetLayout()
    import('/lua/modules/ui/game/helptext.lua').SetLayout()
    import('/lua/modules/ui/game/score.lua').SetLayout()
end

function OnFirstUpdate()
    EnableWorldSounds()
    local avatars = GetArmyAvatars()
    if avatars and avatars[1]:IsInCategory("COMMAND") then
        local armiesInfo = GetArmiesTable()
        local focusArmy = armiesInfo.focusArmy
        local playerName = armiesInfo.armiesTable[focusArmy].nickname
        UISelectAndZoomTo( avatars[1], 0.0 )
        ConExecute('RenameUnit ' .. playerName)
    end
    import('/lua/modules/UserMusic.lua').StartPeaceMusic()
    import('/lua/modules/ui/game/score.lua').CreateScoreOverlay()
    PlaySound( Sound { Bank='AmbientTest', Cue='AMB_Planet_Rumble_zoom'} )
    ForkThread( 
        function()
            WaitSeconds(3.0)
            FlushEvents()
            import('/lua/modules/ui/game/worldview.lua').UnlockInput()
        end
    )
end

function CreateUI(isReplay)
    local prefetchTable = { models = {}, anims = {}, d3d_textures = {}, batch_textures = {} }

    -- set up our layout change function
    UIUtil.changeLayoutFunction = SetLayout

    -- update loc table with player's name
    local focusarmy = GetFocusArmy()
    if focusarmy >= 1 then
        LocGlobals.PlayerName = GetArmiesTable().armiesTable[focusarmy].nickname
    end

    GameCommon.InitializeUnitIconBitmaps(prefetchTable.batch_textures)

    gameParent = UIUtil.CreateScreenGroup(GetFrame(0), "GameMain ScreenGroup")

    controlClusterGroup, statusClusterGroup, mapGroup =  import('/lua/modules/ui/game/borders.lua').SetupBorderControl(gameParent)

    controlClusterGroup:SetNeedsFrameUpdate(true)
    controlClusterGroup.OnFrame = function(self, deltaTime)
        controlClusterGroup:SetNeedsFrameUpdate(false)
        OnFirstUpdate()
    end

    import('/lua/modules/ui/game/worldview.lua').CreateMainWorldView(gameParent, mapGroup)
    import('/lua/modules/ui/game/worldview.lua').LockInput()

    local massGroup, energyGroup = import('/lua/modules/ui/game/economy.lua').CreateEconomyBar(statusClusterGroup)
    import('/lua/modules/ui/game/options.lua').CreateOptionsPanel(statusClusterGroup, massGroup, energyGroup)

    mfdControl = import('/lua/modules/ui/game/multifunction.lua').SetupMultifunctionControl(controlClusterGroup)
    if isReplay then
        ordersControl = import('/lua/modules/ui/game/replay.lua').SetupReplayControl(controlClusterGroup, mfdControl)
    else
        ordersControl = import('/lua/modules/ui/game/orders.lua').SetupOrdersControl(controlClusterGroup, mfdControl)
    end
    queueControl = import('/lua/modules/ui/game/queue.lua').SetupQueueControl(controlClusterGroup, mfdControl, ordersControl)
    import('/lua/modules/ui/game/construction.lua').SetupConstructionControl(controlClusterGroup, mfdControl, ordersControl, queueControl)
    import('/lua/modules/ui/game/selectionmanager.lua').SetupSelectionBox(controlClusterGroup, mfdControl, ordersControl, queueControl)
    import('/lua/modules/ui/game/unitview.lua').SetupUnitViewLayout(controlClusterGroup)
    import('/lua/modules/ui/game/unitviewDetail.lua').SetupUnitViewLayout(mapGroup, mapGroup)
    import('/lua/modules/ui/game/chat.lua').SetupChatLayout(mapGroup)
    import('/lua/modules/ui/game/avatars.lua').CreateAvatarIcons(mapGroup)
    import('/lua/modules/ui/game/transmissionlog.lua').CreateTransmissionLog()
    import('/lua/modules/ui/game/missiontext.lua').CreateMissionText(mapGroup, massGroup, energyGroup)
    import('/lua/modules/ui/game/objectives2.lua').SetupObjectivesLayout(mapGroup)
    import('/lua/modules/ui/game/objectiveDetail.lua').SetupObjectiveDetail(mapGroup)
    import('/lua/modules/ui/game/helptext.lua').CreateHelpText(mapGroup)
    import('/lua/modules/ui/game/timer.lua').CreateTimerDialog(mapGroup)
    import('/lua/modules/ui/game/pause.lua').CreatePauseUI(mapGroup)
    import('/lua/modules/ui/game/consoleecho.lua').CreateConsoleEcho(mapGroup)
    import('/lua/modules/ui/game/enhancements.lua').CreateDialog(UIUtil.currentLayout, mfdControl, gameParent, mapGroup)

    import('/lua/modules/ui/game/diplomacy.lua').CreateDiplomacyControl()

    if GetNumRootFrames() > 1 then
        import('/lua/modules/ui/game/multihead.lua').CreateSecondView()
    end

    controlClusterGroup.HandleEvent = function(self, event)
        if event.Type == "WheelRotation" then
            import('/lua/modules/ui/game/worldview.lua').ForwardMouseWheelInput(event)
            return true
        end
        return false
    end

    statusClusterGroup.HandleEvent = function(self, event)
        if event.Type == "WheelRotation" then
            import('/lua/modules/ui/game/worldview.lua').ForwardMouseWheelInput(event)
            return true
        end
        return false
    end
    
    Prefetcher:Update(prefetchTable)
end

local provider = false

local function LoadDialog(parent)
    local background = Bitmap(parent, UIUtil.UIFile('/portal02.dds'))
    LayoutHelpers.FillParent(background, parent)
    background.Depth:Set(GetFrame(parent:GetRootFrame():GetTargetHead()):GetTopmostDepth() + 1)
    
    local movie = Movie(background, "/movies/FMV_loading02.sfd")
    LayoutHelpers.AtCenterIn(movie, parent)
    movie:Loop(true)
    movie:Play()

    local text = '::  ' .. LOC('<LOC LOAD_0000>IN TRANSIT') .. '  ::'
    local textControl = UIUtil.CreateText(background, text, 20, UIUtil.bodyFont)
    textControl:SetColor('2cffff')
    LayoutHelpers.AtHorizontalCenterIn(textControl, parent)
    LayoutHelpers.AtBottomIn(textControl, movie, -20)
    import('/lua/modules/maui/effecthelpers.lua').Pulse(textControl, 1, 0, .8)

    return background
end

function CreateWldUIProvider()

    provider = WldUIProvider()

    local loadingDialog = false
    local waitingDialog = false
    local frame1Logo = false

    local lastTime = 0

    provider.StartLoadingDialog = function(self)
		GetCursor():Hide()
		supressExitDialog = true
        if not loadingDialog then
            loadingDialog = LoadDialog(GetFrame(0))
            if GetNumRootFrames() > 1 then
                local frame1 = GetFrame(1)
                local frame1Logo = Bitmap(frame1, UIUtil.UIFile('/marketing/splash.dds'))
                LayoutHelpers.FillParent(frame1Logo, frame1)
            end
        end
    end

    provider.UpdateLoadingDialog = function(self, elapsedTime)
        if loadingDialog then
        end
    end

    provider.StopLoadingDialog = function(self)
		GetCursor():Show()
        if loadingDialog then
            loadingDialog:Destroy()
            loadingDialog = false
        end
        FlushEvents()
    end

    provider.StartWaitingDialog = function(self)
        if not waitingDialog then waitingDialog = UIUtil.ShowInfoDialog(GetFrame(0), "<LOC gamemain_0001>Waiting For Other Players...") end
    end

    provider.UpdateWaitingDialog = function(self, elapsedTime)
        -- currently no function, but could animate waiting dialog
    end

    provider.StopWaitingDialog = function(self)
        if waitingDialog then
            waitingDialog:Destroy()
            waitingDialog = false
        end
        FlushEvents()
    end

    provider.CreateGameInterface = function(self, inIsReplay)
        isReplay = inIsReplay
        if frame1Logo then
            frame1Logo:Destroy()
            frame1Logo = false
        end
        CreateUI(isReplay)
		supressExitDialog = false
        FlushEvents()
    end

    provider.DestroyGameInterface = function(self)
        if gameParent then gameParent:Destroy() end
    end
end

-- This function is called whenever the set of currently selected units changes
-- See /lua/modules/unit.lua for more information on the lua unit object
--      oldSelection: What the selection was before
--      newSelection: What the selection is now
--      added: Which units were added to the old selection
--      removed: Which units where removed from the old selection
function OnSelectionChanged(oldSelection, newSelection, added, removed)

    local availableOrders,  buildableCategories = GetUnitCommandData(newSelection)
    local isOldSelection = table.equal(oldSelection, newSelection)

    if not gameUIHidden then
        import('/lua/modules/ui/game/queue.lua').CheckForOrderQueue(newSelection, newSelection)
        if not isReplay then
            import('/lua/modules/ui/game/orders.lua').SetAvailableOrders(availableOrders, newSelection)
        end
        -- todo change the current command mode if no longer available? or set to nil?
        import('/lua/modules/ui/game/selectionmanager.lua').SelectionManager(buildableCategories,newSelection)
    end
        
    if not isOldSelection then
        if not gameUIHidden then
            import('/lua/modules/ui/game/enhancements.lua').ToggleDialog(nil, nil, true)
        end
        import('/lua/modules/ui/game/selection.lua').PlaySelectionSound(added)
    end

end

function OnQueueChanged(newQueue)
    if not gameUIHidden then
        import('/lua/modules/ui/game/queue.lua').OnQueueChanged(newQueue)
    end
end

# Called after the Sim has confirmed the game is indeed paused. This will happen
# on everyone's machine in a network game.
function OnPause(pausedBy, timeoutsRemaining)
    local isOwner = false
    if pausedBy == SessionGetLocalCommandSource() then
        isOwner = true
    end
    import('/lua/modules/ui/game/pause.lua').OnPause(pausedBy, timeoutsRemaining, isOwner)
    if not SessionIsReplay() then
        import('/lua/modules/ui/game/options.lua').SetPauseButton(true, isOwner)
    end
end

# Called after the Sim has confirmed that the game has resumed.
function OnResume()
    import('/lua/modules/ui/game/pause.lua').OnResume()
    if not SessionIsReplay() then
        import('/lua/modules/ui/game/options.lua').SetPauseButton(false)
    end
end

# Called immediately when the user hits the pause button. This only ever gets
# called on the machine that initiated the pause (i.e. other network players
# won't call this)
function OnUserPause(pause)
    local likelyTimeouts = import('/lua/modules/ui/game/pause.lua').GetLikelyTimeouts()
    local focus = GetArmiesTable().focusArmy
    if likelyTimeouts != 0 then
        if focus == -1 and not SessionIsReplay() then
            return
        end
        PauseSound("World",pause)
        PauseSound("Music",pause)
        PauseVoice("VO",pause)

        if pause then
            import('/lua/modules/ui/game/missiontext.lua').PauseTransmission()
        else
            import('/lua/modules/ui/game/missiontext.lua').ResumeTransmission()
        end
        if SessionIsReplay() then
            import('/lua/modules/ui/game/options.lua').SetPauseButton(pause,true)
        end
    end
end

local _beatFunctions = {}

function AddBeatFunction(fn)
    table.insert(_beatFunctions, fn)
end

function RemoveBeatFunction(fn)
    for i,v in _beatFunctions do
        if v == fn then
            table.remove(_beatFunctions, i)
            break
        end
    end
end

-- this function is called whenever the sim beats
function OnBeat()
    for i,v in _beatFunctions do
        if v then v() end
    end
end

function GetStatusCluster()
    return statusClusterGroup
end

function GetControlCluster()
    return controlClusterGroup
end

function GetGameParent()
    return gameParent
end

function HideGameUI(movieBool)
    if gameParent then
        if gameUIHidden then
            gameUIHidden = false
            controlClusterGroup:Show()
            statusClusterGroup:Show()
            import('/lua/modules/ui/game/worldview.lua').Contract()
            import('/lua/modules/ui/game/borders.lua').HideBorder(false)
            import('/lua/modules/ui/game/tracking.lua').Expand()
            import('/lua/modules/ui/game/unitview.lua').Expand()
            import('/lua/modules/ui/game/avatars.lua').Expand()
            import('/lua/modules/ui/game/economy.lua').Expand()
            import('/lua/modules/ui/game/pause.lua').Expand()
            import('/lua/modules/ui/game/score.lua').Expand()
            import('/lua/modules/ui/game/objectives2.lua').Expand()
            import('/lua/modules/ui/game/multifunction.lua').Expand()
            import('/lua/modules/ui/game/unitviewDetail.lua').Expand()
        else
            gameUIHidden = true
            controlClusterGroup:Hide()
            statusClusterGroup:Hide()
            import('/lua/modules/ui/game/worldview.lua').Expand()
            import('/lua/modules/ui/game/borders.lua').HideBorder(true)
            import('/lua/modules/ui/game/tracking.lua').Contract()
            import('/lua/modules/ui/game/enhancements.lua').Contract()
            import('/lua/modules/ui/game/unitview.lua').Contract()
            import('/lua/modules/ui/game/unitviewDetail.lua').Contract()
            import('/lua/modules/ui/game/avatars.lua').Contract()
            import('/lua/modules/ui/game/economy.lua').Contract()
            import('/lua/modules/ui/game/pause.lua').Contract()
            import('/lua/modules/ui/game/score.lua').Contract()
            import('/lua/modules/ui/game/objectives2.lua').Contract()
            import('/lua/modules/ui/game/multifunction.lua').Contract()
        end
    end
end

# Given a userunit that is adjacent to a given blueprint, does it yield a
# bonus? Used by the UI to draw extra info
function OnDetectAdjacencyBonus(userUnit, otherBp)
    # fixme: todo
    return true
end

function OnFocusArmyUnitDamaged(unit)
    import('/lua/modules/UserMusic.lua').NotifyBattle()
end

