--*****************************************************************************
--* File: lua/modules/ui/game/unitview.lua
--* Author: Chris Blackwell
--* Summary: Rollover unit view control
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local StatusBar = import('/lua/modules/maui/statusbar.lua').StatusBar

local rolloverInfo = false
local consTrue = false
local focusBool = false

local showBool = true

numRoDisplayFields = 3

controls = {
    roInfoLine = {},
    roInfoLineBG = {},
    roInfoLine2 = {},
    parent = false,
    rolloverGroup = false,
    roBG = false,
    roIcon = false,
    roStratIcon = false,
    roVeteranIcons = {},
    roDisplay = {},
    roVeteranIconParent = false,
    roTextGroup = false,
    roTextGroupBG = false,
    roName = false,
    roId = false,
    roHealth = false,
    roHealthBar = false,
    roFocusArrow = false,
    roFocusName = false,
    roFocusIcon = false,
    roFocusHealthBar = false,
    roFocusHealthPercent = false,
    roTechLevelPips = {},
    AbilityList = false,
}

function Contract()
    showBool = false
end

function Expand()
    showBool = true
end

function SetZoomAlpha()
    local view = GetCamera('WorldCamera')
    
    local minZoom = view:GetMinZoom()
    local maxZoom = view:GetMaxZoom()
    local curZoom = view:GetZoom()
    
    if (curZoom / (maxZoom - minZoom)) > .9 then
        controls.rolloverGroup:SetAlpha(.5, true)
    else
        controls.rolloverGroup:SetAlpha(1, true)
    end
end

function ShowROBox(state, focusBool, inDisplayIndex, progBar, showAbilities)
    if(state) then
        controls.rolloverGroup:Show()
        controls.rolloverGroup:SetAlpha(1, true)
    else
        controls.rolloverGroup:Hide()
        consTrue = false
    end
    if not progBar then
        controls.roProgBar:Hide()
    end
    if not showAbilities and controls.AbilityList then
        controls.AbilityList:Hide()
    end
    if focusBool then
        if controls.roFocusBG then
            controls.roFocusBG:Show()
            controls.roFocusHealthBar:Show()
            controls.roFocusName:Show()
        end
        controls.roFocusIcon:Show()
        controls.roFocusHealthPercent:Show()
    else
        if controls.roFocusBG then
            controls.roFocusHealthBar:Hide()
            controls.roFocusBG:Hide()
            controls.roFocusName:Hide()
        end
        controls.roFocusIcon:Hide()
        controls.roFocusHealthPercent:Hide()
        controls.roFocusIcon:SetSolidColor("00000000")
    end
end

function NetResources(produced, requested)
    local netReturn = produced - requested
    if(netReturn >= 0) then
        return "+" .. tostring(netReturn)
    else
        return netReturn
    end
end

function UpdateVetIcons(level, faction)
    local factionData = import('/lua/factions.lua')

    for i = 1, 5 do
        if faction and faction != '' then
            controls.roVeteranIcons[i]:SetTexture(UIUtil.UIFile(factionData.Factions[factionData.FactionIndexMap[faction]].VeteranIcon))
        end
        controls.roVeteranIcons[i]:Hide()
    end
    for i = 1, level do
        controls.roVeteranIcons[i]:Show()
    end
end

function FormatTime(seconds)
    local tempSeconds = math.floor(seconds)
    local tempMinutes = 00

    tempMinutes = math.floor(tempSeconds / 60)
    tempSeconds = tempSeconds - (tempMinutes * 60)
    if(tempMinutes < 10) then
        tempMinutes = "0" .. tostring(tempMinutes)
    end
    if(tempSeconds < 10) then
        tempSeconds = "0" .. tostring(tempSeconds)
    end
    local tempTime = tostring(tempMinutes) .. ":" .. tostring(tempSeconds)
    return tempTime
end

function UpdateROInfo(bp, constructionBool, buildingUnit, enhancementBool)
    local vetLevel = 0
    focusBool = false
    if bp then
        local displayIndex = 1
        if enhancementBool then
            controls.roName:SetText(bp.Name)
            controls.roIcon:SetTexture(UIUtil.UIFile('/icons/units/default_icon.dds'))
            controls.roStratIcon:Hide()
            controls.roName:SetColor("FFFF9E06")
            controls.roId:SetColor("FF06ECEC")
            controls.roHealth:SetText("")
            controls.roHealthBar:SetValue(0)
            controls.roHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay['mass'], tostring(math.ceil(bp.BuildCostMass)), 'FF00FF00', 2, false)
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay['energy'], tostring(math.ceil(bp.BuildCostEnergy)), 'FF00FF00', 3, false)
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[1], FormatTime(bp.BuildTime), 'FF00FF00', 5, false)
            UpdateVetIcons(0)
            consTrue = true
            ShowROBox(true, focusBool, displayIndex)
        else
            local techLevel = 0
            for i, v in bp.Categories do
                if v == "TECH1" then
                    techLevel = 1
                end
                if v == "TECH2" then
                    techLevel = 2
                end
                if v == "TECH3" then
                    techLevel = 3
                end
            end
            for i = 1, 3 do
                if i <= techLevel then
                    controls.roTechLevelPips[i]:SetTexture(UIUtil.UIFile('/game/icon_tech-level/tech-level_solid.dds'))
                else
                    controls.roTechLevelPips[i]:SetTexture(UIUtil.UIFile('/game/icon_tech-level/tech-level_open.dds'))
                end
            end
            local iconName = GameCommon.GetCachedUnitIconFileNames(bp)
            controls.roIcon:SetTexture(iconName)
            if (bp.StrategicIconName) then
                if DiskGetFileInfo("/textures/ui/common/game/strategicicons/" .. string.lower(bp.StrategicIconName) .. "_over.dds") then
                    controls.roStratIcon:SetTexture("/textures/ui/common/game/strategicicons/" .. string.lower(bp.StrategicIconName) .. "_over.dds")
                    controls.roStratIcon:Show()
                else
                    controls.roStratIcon:SetSolidColor('00000000')
                end
            end
            
            local civilian = false
            for i, v in bp.Categories do
                if v == 'CIVILIAN' then
                    civilian = true
                end
            end
            if civilian then
                if rolloverInfo.customName or bp.General.UnitName then
                    controls.roName:SetText(LOCF("%s", rolloverInfo.customName or bp.General.UnitName))
                end
            else
                if techLevel == 0 then
                    if rolloverInfo.customName or bp.General.UnitName then
                        controls.roName:SetText(LOCF("%s: %s", rolloverInfo.customName or bp.General.UnitName, bp.Description))
                    else
                        controls.roName:SetText(LOCF("%s", bp.Description))
                    end
                else
                    if rolloverInfo.customName or bp.General.UnitName then
                        controls.roName:SetText(LOCF("%s: Tech %d %s", rolloverInfo.customName or bp.General.UnitName, techLevel, bp.Description))
                    else
                        controls.roName:SetText(LOCF("Tech %d %s", techLevel, bp.Description))
                    end
                end
            end

            local function UpdateStatusColor(control, currentval, maxval)
                if control._rangeMax != maxval then
                    control:SetRange(0, maxval)
                end
                if currentval / maxval > .75 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
                elseif currentval / maxval > .25 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'))
                elseif currentval / maxval > 0 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                end
                control:SetValue(currentval)
            end
            controls.roName:SetColor("FFFF9E06")
            controls.roId:SetColor("FF06ECEC")
            
            
            if(constructionBool) then
                controls.roHealth:SetText(string.format("%d/%d", bp.Defense.MaxHealth, bp.Defense.MaxHealth))
                controls.roHealth:SetColor("FF02f3e2")
                if controls.roHealthBar._rangeMax != bp.Defense.MaxHealth then
                    controls.roHealthBar:SetRange(0, bp.Defense.MaxHealth)
                end
                controls.roHealthBar:SetValue(bp.Defense.MaxHealth)
            else
                UpdateStatusColor(controls.roHealthBar, rolloverInfo.health, rolloverInfo.maxHealth)
                controls.roHealth:SetText(string.format("%d/%d", rolloverInfo.health, rolloverInfo.maxHealth))
                controls.roHealth:SetColor("FFF0F0F0")
            end
            if(constructionBool) then
            else
                if rolloverInfo.nukeSiloMaxStorageCount then
                    if( rolloverInfo.nukeSiloMaxStorageCount > 0 ) then
                        if( rolloverInfo.nukeSiloBuildCount > 0 ) then
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.nukeSiloStorageCount) .. " / " .. tostring(rolloverInfo.nukeSiloMaxStorageCount) .. " (+" .. tostring(rolloverInfo.nukeSiloBuildCount) .. ")", 'FF9eff19', 1, false)
                        else
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.nukeSiloStorageCount) .. " / " .. tostring(rolloverInfo.nukeSiloMaxStorageCount), 'FF9eff19', 1, false)
                        end
                    end
                end
                if rolloverInfo.tacticalSiloMaxStorageCount then
                    if( rolloverInfo.tacticalSiloMaxStorageCount > 0 ) then
                        if( rolloverInfo.tacticalSiloBuildCount > 0 ) then
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.tacticalSiloStorageCount) .. " / " .. tostring(rolloverInfo.tacticalSiloMaxStorageCount) .. " (+" .. tostring(rolloverInfo.tacticalSiloBuildCount) .. ")", 'FF9eff19', 7, false)
                        else
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.tacticalSiloStorageCount) .. " / " .. tostring(rolloverInfo.tacticalSiloMaxStorageCount), 'FF9eff19', 7, false)
                        end
                    end
                end
            end
            local showAbilities = false
            if controls.AbilityList then
                if bp.Display.Abilities != nil then
                    showAbilities = true
                    for i, v in bp.Display.Abilities do
                        local index = i
                        if controls.AbilityList.Value[index] then
                            controls.AbilityList.Value[index]:SetText(LOC(v))
                        else
                            controls.AbilityList.Value[index] = UIUtil.CreateText( controls.AbilityList, LOC(v), 12, UIUtil.bodyFont)
                            LayoutHelpers.Below(controls.AbilityList.Value[index], controls.AbilityList.Value[index-1])
                        end
                    end
                    local maxWidth = 0
                    for i, v in controls.AbilityList.Value do
                        local index = i
                        if v.Width() > maxWidth then
                            maxWidth = v.Width()
                        end
                        if index > table.getsize(bp.Display.Abilities) then
                            v:SetText("")
                        end
                    end
                    controls.AbilityList.bgmidmiddle.Width:Set(maxWidth)
                    controls.AbilityList.bgmidmiddle.Height:Set(function() return (table.getsize(bp.Display.Abilities) * controls.AbilityList.Value[1].Height()) - 10  end)
                else
                    controls.AbilityList:Hide()
                    for i, v in controls.AbilityList.Value do
                        v:SetText("")
                    end
                end
            end
            if(constructionBool) then
                consTrue = true
                if(buildingUnit) then
                    local timeCost, energyCost, massCost = import('/lua/modules/game.lua').GetConstructEconomyModel(buildingUnit, bp.Economy)
                    if(massCost) then
                        SetDisplayLine(displayIndex, controls.roDisplay['mass'], math.ceil(massCost), "FF00FF00", 2, false)
                    end
                    if(energyCost) then
                        SetDisplayLine(displayIndex, controls.roDisplay['energy'], math.ceil(energyCost), "FF00FF00", 3, false)
                    end
                    if(timeCost) then
                        displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], FormatTime(timeCost), "FF00FF00", 5, false)
                    end
                end
            else
                if(rolloverInfo.massProduced >= 0 and rolloverInfo.massConsumed >= 0) then
                    SetDisplayLine(displayIndex, controls.roDisplay['mass'], NetResources(rolloverInfo.massProduced, rolloverInfo.massConsumed), 'FF00FF00', 2, false)
                else                    
                    SetDisplayLine(displayIndex, controls.roDisplay['mass'], '', 'FF00FF00', 2, false)
                end
                if(rolloverInfo.energyProduced >= 0 and rolloverInfo.energyConsumed >= 0) then
                    SetDisplayLine(displayIndex, controls.roDisplay['energy'], NetResources(rolloverInfo.energyProduced, rolloverInfo.energyConsumed), 'FF00FF00', 3, false)
                else
                    SetDisplayLine(displayIndex, controls.roDisplay['energy'], '', 'FF00FF00', 3, false)
                end
                if(rolloverInfo.kills != 0 and rolloverInfo.kills != nil) then
                    local vetLevel5 = 1000
                    local vetLevel4 = 500
                    local vetLevel3 = 250
                    local vetLevel2 = 100
                    local vetLevel1 = 25
                    if bp.Veteran then
                        vetLevel5 = bp.Veteran['Level5']
                        vetLevel4 = bp.Veteran['Level4']
                        vetLevel3 = bp.Veteran['Level3']
                        vetLevel2 = bp.Veteran['Level2']
                        vetLevel1 = bp.Veteran['Level1']
                    end
                     
                    displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], rolloverInfo.kills, 'FFFF0000', 4, false)
                    if(rolloverInfo.kills >= vetLevel5) then
                        vetLevel = 5
                    elseif(rolloverInfo.kills >= vetLevel4) then
                        vetLevel = 4
                    elseif(rolloverInfo.kills >= vetLevel3) then
                        vetLevel = 3
                    elseif(rolloverInfo.kills >= vetLevel2) then
                        vetLevel = 2
                    elseif(rolloverInfo.kills >= vetLevel1) then
                        vetLevel = 1
                    else
                        vetLevel = 0
                    end
                end
            end
            local focusBP
            if rolloverInfo.focus then
                if __blueprints[rolloverInfo.focus.blueprintId] then
                    focusBP = __blueprints[rolloverInfo.focus.blueprintId]
                    if controls.roFocusBG then
                        if focusBP.General.UnitName != nil then
                            controls.roFocusName:SetText(LOC(focusBP.General.UnitName))
                        else
                            controls.roFocusName:SetText(LOC(focusBP.Description))
                        end
                        UpdateStatusColor(controls.roFocusHealthBar, rolloverInfo.focus.health, rolloverInfo.focus.maxHealth)
                    end
                    local iconFocusName = GameCommon.GetCachedUnitIconFileNames(focusBP)
                    controls.roFocusIcon:SetTexture(iconFocusName)
                    controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focus.health / rolloverInfo.focus.maxHealth)) .. "%")
                    controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                    focusBool = true
                end
            elseif rolloverInfo.focusUpgrade then
                if __blueprints[rolloverInfo.focusUpgrade.blueprintId] then
                    focusBP = __blueprints[rolloverInfo.focusUpgrade.blueprintId]
                    if controls.roFocusBG then
                        local techString = ''
                        for i, v in focusBP.Categories do
                            if v == "TECH2" then
                                techString = '<LOC _Tech_2>'
                            elseif v == "TECH3" then
                                techString = '<LOC _Tech_3>'
                            end
                        end
                        controls.roFocusName:SetText(LOCF('%s %s', "<LOC _Upgrading_to>", techString))
                        UpdateStatusColor(controls.roFocusHealthBar, rolloverInfo.focusUpgrade.health, rolloverInfo.focusUpgrade.maxHealth)
                    end
                    local iconFocusName = GameCommon.GetCachedUnitIconFileNames(focusBP)
                    controls.roFocusIcon:SetTexture(iconFocusName)
                    controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focusUpgrade.health / rolloverInfo.focusUpgrade.maxHealth)) .. "%")
                    controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                    focusBool = true
                end
            elseif rolloverInfo.focusProp then
                if __blueprints[rolloverInfo.focusProp.blueprintId] then
                    focusBP = __blueprints[rolloverInfo.focusProp.blueprintId]
                    if controls.roFocusBG then
                        controls.roFocusName:SetText(LOC("<LOC rollover_0000>Reclaiming"))
                        controls.roFocusHealthBar:SetRange(0, 1.0)
                        controls.roFocusHealthBar:SetValue(rolloverInfo.focusProp.fractionComplete)
                        controls.roFocusHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                    end
                    controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focusProp.fractionComplete / 1.0)) .. "%")
                    controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                    controls.roFocusIcon:SetTexture(UIUtil.UIFile('/game/orders/reclaim_btn_up.dds'))
                    focusBool = true
                end
            else
                focusBool = false
            end
            local showprogBar = false
            if displayIndex < numRoDisplayFields then
                displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], string.format("%d%%", math.floor((rolloverInfo.health / rolloverInfo.maxHealth) * 100)), 'FF00FF00', 8, true)
            end
            if rolloverInfo.fuelRatio > 0 then
                showprogBar = true
                controls.roProgBar:SetValue(rolloverInfo.fuelRatio)
                if displayIndex < numRoDisplayFields then
                    displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], string.format("%d%%", math.ceil(rolloverInfo.fuelRatio * 100)), 'ffffd300', 9, true)
                end
            elseif rolloverInfo.workProgress > 0 then
                showprogBar = true
                controls.roProgBar:SetValue(rolloverInfo.workProgress)
                if displayIndex < numRoDisplayFields then
                    displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], string.format("%d%%", math.ceil(rolloverInfo.workProgress * 100)), 'ffffd300', 10, true)
                end
            end
            ShowROBox(true, focusBool, displayIndex, showprogBar, showAbilities)
            #SetZoomAlpha()
            local faction = ''
            local factionData = import('/lua/factions.lua')
            for i, v in bp.Categories do
                local lcCat = string.lower(v)
                if factionData.FactionIndexMap[lcCat] then
                    faction = lcCat
                    break
                end
            end
            UpdateVetIcons(vetLevel, faction)
            while displayIndex <= numRoDisplayFields do
                displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], '', 'FF000000', 6, true)
            end
        end
    else
        if(consTrue) then
            ShowROBox(true, focusBool)
            UpdateVetIcons(vetLevel)
        else
            ShowROBox(false, focusBool)
        end
    end
end

function SetDisplayLine(displayIndex, control, text, color, iconType, hidden)
    control.text:SetText(text)
    local textColor = color
    if tonumber(string.sub(text, 1)) then
        if tonumber(string.sub(text, 1)) >= 0 then
            textColor = 'FF00FF00'
        else
            textColor = 'FFFF0000'
        end
    end
    control.text.Left:Set(function() return control.icon.Right() + 2 end)
    control.text:SetColor(textColor)
    if iconType == 1 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-nuke_bmp.dds'))
    elseif iconType == 2 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-mass_bmp.dds'))
    elseif iconType == 3 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-energy_bmp.dds'))
    elseif iconType == 4 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-skull_bmp.dds'))
    elseif iconType == 5 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-clock_bmp.dds'))
    elseif iconType == 6 then
        control.icon:SetSolidColor('00000000')
    elseif iconType == 7 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit_view_icons/tactical.dds'))
    elseif iconType == 8 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit_view_icons/redcross.dds'))
    elseif iconType == 9 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit_view_icons/fuel.dds'))
    elseif iconType == 10 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit_view_icons/build.dds'))
    end
    if hidden then
        control.text:Hide()
        control.icon:Hide()
    else
        control.text:Show()
        control.icon:Show()
    end
    local tempIndex = displayIndex + 1
    return tempIndex
end

function CreateTextbox(parent)
    local group = Group(parent)
    
    group.bgtopleft = Bitmap(group)
    group.bgtopmiddle = Bitmap(group)
    group.bgtopright = Bitmap(group)
    
    group.bgmidleft = Bitmap(group)
    group.bgmidmiddle = Bitmap(group)
    group.bgmidright = Bitmap(group)
    
    group.bgbotleft = Bitmap(group)
    group.bgbotmiddle = Bitmap(group)
    group.bgbotright = Bitmap(group)
    
    group.bgtopleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ul.dds'))
    group.bgtopmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_horz_um.dds'))
    group.bgtopright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ur.dds'))
    
    group.bgmidleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_vert_l.dds'))
    group.bgmidmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_m.dds'))
    group.bgmidright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_vert_r.dds'))
    
    group.bgbotleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ll.dds'))
    group.bgbotmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_lm.dds'))
    group.bgbotright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_lr.dds'))

    group.Value = {}
    group.Value[1] = UIUtil.CreateText( group, "", 12, UIUtil.bodyFont)
    
    return group
end

function CommonLogic()
    if controls.rolloverGroup then
        -- get frame updates so we can check what's under the cursor
        controls.rolloverGroup:DisableHitTest(true)
        controls.rolloverGroup:SetNeedsFrameUpdate(true)
        controls.rolloverGroup.OnFrame = function(self, elapsedTime)
            -- supress rollover if UI hidden
            if import('/lua/modules/ui/game/gamemain.lua').gameUIHidden then return end
            rolloverInfo = GetRolloverInfo()
            local bp = nil
            if rolloverInfo then
                controls.rolloverGroup:SetAlpha(1, true)
                bp = __blueprints[rolloverInfo.blueprintId]
            else
                local cam = GetCamera('WorldCamera')
                local camMax = cam:GetMaxZoom()
                local curZoom = cam:GetZoom() / camMax
                if curZoom > .25 then
                    controls.rolloverGroup:SetAlpha(1.1 - curZoom, true)
                else
                    controls.rolloverGroup:SetAlpha(1, true)
                end
            end
            if showBool then
                UpdateROInfo(bp, false)
            end
        end
    end
end

function SetLayout(layout)
    if not controls.rolloverGroup then
        controls.rolloverGroup = Group(controls.parent, "Unit Rollover Group")
    end
    import(UIUtil.GetLayoutFilename('unitview')).SetLayout()
    CommonLogic()
end

function SetupUnitViewLayout(orderControl)
    controls.parent = orderControl
    SetLayout(UIUtil.currentLayout)
end