
local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Prefs = import('/lua/modules/ui/prefs.lua')
local UnitDescriptions = import('/lua/modules/ui/help/unitdescription.lua').Description

View = false
ViewState = "full"
MapView = false

function Contract()
    View:Hide()
end

function Expand()
end

function GetTechLevelString(bp)
    if EntityCategoryContains(categories.TECH1, bp.BlueprintId) then
        return 1
    elseif EntityCategoryContains(categories.TECH2, bp.BlueprintId) then
        return 2
    elseif EntityCategoryContains(categories.TECH3, bp.BlueprintId) then
        return 3
    elseif EntityCategoryContains(categories.EXPERIMENTAL, bp.BlueprintId) then
        return 4
    else
        return 3
    end
end

function FormatTime(seconds)
    local tempSeconds = math.floor(seconds)
    local tempMinutes = 00

    tempMinutes = math.floor(tempSeconds / 60)
    tempSeconds = tempSeconds - (tempMinutes * 60)
    if(tempMinutes < 10) then
        tempMinutes = "0" .. tostring(tempMinutes)
    end
    if(tempSeconds < 10) then
        tempSeconds = "0" .. tostring(tempSeconds)
    end
    local tempTime = tostring(tempMinutes) .. ":" .. tostring(tempSeconds)
    return tempTime
end

function GetAbilityList(bp)
    local abilitiesList = {}

    return abilitiesList
end
    
function CheckFormat()
    if ViewState != Prefs.GetOption('uvd_format') then
        SetLayout()
    end
    if ViewState == "off" then
        return false
    else
        return true
    end
end
    
function ShowView(showUpKeep, enhancement, showecon, showAbilities, showShield)
    import('/lua/modules/ui/game/unitview.lua').ShowROBox(false, false)
    View:Show()
    if not showUpKeep then
        View.UpkeepGroup:Hide()
    end
    if not showecon then
        View.BuildCostGroup:Hide()
        View.UpkeepGroup:Hide()
        View.TimeStat:Hide()
        View.HealthStat:Hide()
    end
    if enhancement then
        View.HealthStat:Hide()
        if View.AbilityList then
            View.AbilityList:Hide()
        end
    end
    if not showShield then
        View.ShieldStat:Hide()
    end
    if View.Description then
        if not showAbilities then
            View.AbilityList:Hide()
        end
        if ViewState == "limited" or View.Description.Value[1]:GetText() == "" then
            View.Description:Hide()
        end
    end
end
    
function ShowEnhancement(bp, bpID, iconID, userUnit)
    if CheckFormat() then
        # Name / Description
        local tempBmpName = ''
        --LOG("Warning: ShowE ", bp.Slot)
        if bp.Slot == "Back" then
            tempBmpName = bpID .. "-b-" .. iconID
        elseif bp.Slot == "LCH" then
            tempBmpName = bpID .. "-la-" .. iconID
        elseif bp.Slot == "RCH" then
            tempBmpName = bpID .. "-ra-" .. iconID
        
        elseif bp.Slot == "Shield" then
            tempBmpName = bpID .. "-ch"
      	else 
      		tempBmpName = bpID .. "-" .. bp.Slot .. "-" .. iconID
        end
      	
        tempBmpName = tempBmpName .. '_btn_up.dds'
        if string.sub(bpID, 2, 2) == 'a' then
            tempBmpName = '/dialogs/enhance-icons-aeon-' .. bpID .. '/'.. tempBmpName
        elseif string.sub(bpID, 2, 2) == 'e' then
            tempBmpName = '/dialogs/enhance-icons-uef-' .. bpID .. '/' .. tempBmpName
        elseif string.sub(bpID, 2, 2) == 'r' then
            tempBmpName = '/dialogs/enhance-icons-cybran-' .. bpID .. '/' .. tempBmpName
        end

       --if DiskGetFileInfo(UIUtil.UIFile(tempBmpName)) == false then
       --     tempBmpName = '/customIcon/icon_btn_up.dds'
       --end
        View.UnitImg:SetTexture(UIUtil.UIFile(tempBmpName))
        View.UnitImg.Height:Set(52)
        View.UnitImg.Width:Set(52) 
        
        if bp.Name != nil then
            View.UnitName:SetText(LOC(bp.Name))
        else
            View.UnitName:SetText("")
        end
        View.UnitShortDesc:SetText(LOC(bp.Slot))
        
        
        local showecon = true
        local showAbilities = false
        local showUpKeep = false
        local time, energy, mass
        if bp.Icon != nil and not string.find(bp.Name, 'Remove') then
            time, energy, mass = import('/lua/modules/game.lua').GetConstructEconomyModel(userUnit, bp)
            time = math.max(time, 1)
            showUpKeep = DisplayResources(bp, time, energy, mass)
            View.TimeStat.Value:SetText(string.format("%s", FormatTime(time)))
        else
            showecon = false
            if View.Description then
                View.Description:Hide()
                for i, v in View.Description.Value do
                    v:SetText("")
                end
            end
        end
        
        
        if View.Description then
            local tempDescID = bpID.."-"..iconID
            if UnitDescriptions[tempDescID] and not string.find(bp.Name, 'Remove') then
                local tempDesc = LOC(UnitDescriptions[tempDescID])
                WrapAndPlaceText(tempDesc, View.Description)
            else
                View.Description:Hide()
                for i, v in View.Description.Value do
                    v:SetText("")
                end
            end
        end
        
        local showShield = false
        if bp.ShieldMaxHealth then
            showShield = true
            View.ShieldStat.Value:SetText(bp.ShieldMaxHealth)
        end
    
        ShowView(showUpKeep, true, showecon, showAbilities, showShield)
        if time == 0 and energy == 0 and mass == 0 then
            View.BuildCostGroup:Hide()
            View.TimeStat:Hide()
        end
    else
        View:Hide()
    end
end
    
function WrapAndPlaceText(text, control)
    local lines = import('/lua/modules/maui/text.lua').WrapText(text, control.Value[1].Width(),
            function(text) return control.Value[1]:GetStringAdvance(text) end
        )
    for i, v in lines do
        local index = i
        if control.Value[index] then
            control.Value[index]:SetText(v)
        else
            control.Value[index] = UIUtil.CreateText( control, v, 12, UIUtil.bodyFont)
            LayoutHelpers.Below(control.Value[index], control.Value[index-1])
            control.Value[index].Right:Set(function() return control.Right() - 7 end)
            control.Value[index].Width:Set(function() return control.Right() - control.Left() - 14 end)
            control.Value[index]:SetClipToWidth(true)
        end
        control.BGMid.Height:Set(function() return (math.max(table.getsize(lines), 4) * control.Value[1].Height()) - 50 end)
    end
    for i, v in control.Value do
        local index = i
        if index > table.getsize(lines) then
            v:SetText("")
        end
    end
end
    
function Show(bp, buildingUnit, bpID)
    if CheckFormat() then
        # Name / Description
        if false then
            local foo, iconName = GameCommon.GetCachedUnitIconFileNames(bp)
            if iconName then
                View.UnitIcon:SetTexture(iconName)
            else
                View.UnitIcon:SetTexture(UIUtil.UIFile('/icons/units/default_icon.dds'))    
            end
        end
        if UIUtil.currentLayout != 'minimal' then
            if bp.General.UnitName != nil then
                View.UnitName:SetText(LOC(bp.General.UnitName))
            else
                View.UnitName:SetText(LOC(bp.Description))
            end
            View.UnitShortDesc:SetText(LOCF("Tech %d %s", GetTechLevelString(bp), bp.Description))
        else
            View.UnitName:SetText(LOCF('%s', bp.General.UnitName or bp.Description))
            View.UnitShortDesc:SetText('')
        end
                
        local showecon = true
        local showUpKeep = false
        local showAbilities = false
        if buildingUnit != nil then
            local time, energy, mass = import('/lua/modules/game.lua').GetConstructEconomyModel(buildingUnit, bp.Economy)
            time = math.max(time, 1)
            showUpKeep = DisplayResources(bp, time, energy, mass)
            View.TimeStat.Value:SetText(string.format("%s", FormatTime(time)))
        else
            showecon = false
        end
            
        # Health stat
        View.HealthStat.Value:SetText(string.format("%d", bp.Defense.MaxHealth))
    
        if View.Description then
            if UnitDescriptions[bpID] then
                WrapAndPlaceText(LOC(UnitDescriptions[bpID]), View.Description)
            else
                View.Description:Hide()
                for i, v in View.Description.Value do
                    v:SetText("")
                end
            end
        
            if bp.Display.Abilities != nil then
                showAbilities = true
                for i, v in bp.Display.Abilities do
                    local index = i
                    if View.AbilityList.Value[index] then
                        View.AbilityList.Value[index]:SetText(LOC(v))
                    else
                        View.AbilityList.Value[index] = UIUtil.CreateText( View.AbilityList, LOC(v), 12, UIUtil.bodyFont)
                        LayoutHelpers.Below(View.AbilityList.Value[index], View.AbilityList.Value[index-1])
                    end
                end
                local maxWidth = 0
                for i, v in View.AbilityList.Value do
                    local index = i
                    if v.Width() > maxWidth then
                        maxWidth = v.Width()
                    end
                    if index > table.getsize(bp.Display.Abilities) then
                        v:SetText("")
                    end
                end
                View.AbilityList.bgmidmiddle.Width:Set(maxWidth)
                View.AbilityList.bgmidmiddle.Height:Set(function() return (table.getsize(bp.Display.Abilities) * View.AbilityList.Value[1].Height()) - 10  end)
            else
                View.AbilityList:Hide()
                for i, v in View.AbilityList.Value do
                    v:SetText("")
                end
            end
        end
        local showShield = false
        if bp.Defense.Shield and bp.Defense.Shield.ShieldMaxHealth then
            showShield = true
            View.ShieldStat.Value:SetText(bp.Defense.Shield.ShieldMaxHealth)
        end
        
        local iconName = GameCommon.GetCachedUnitIconFileNames(bp)
        View.UnitImg:SetTexture(iconName)
        View.UnitImg.Height:Set(46)
        View.UnitImg.Width:Set(48)
        
        ShowView(showUpKeep, false, showecon, showAbilities, showShield)
    else
        View:Hide()
    end
end

function DisplayResources(bp, time, energy, mass)
    # Cost Group
    if time > 0 then
        local consumeEnergy = -energy / time
        local consumeMass = -mass / time
        View.BuildCostGroup.EnergyValue:SetText( string.format("%d (%d)",-energy,consumeEnergy) )
        View.BuildCostGroup.MassValue:SetText( string.format("%d (%d)",-mass,consumeMass) )
        
        View.BuildCostGroup.EnergyValue:SetColor( "FFF05050" )
        View.BuildCostGroup.MassValue:SetColor( "FFF05050" )
    end

    # Upkeep Group
    local plusEnergyRate = bp.Economy.ProductionPerSecondEnergy or bp.ProductionPerSecondEnergy
    local negEnergyRate = bp.Economy.MaintenanceConsumptionPerSecondEnergy or bp.MaintenanceConsumptionPerSecondEnergy
    local plusMassRate = bp.Economy.ProductionPerSecondMass or bp.ProductionPerSecondMass
    local negMassRate = bp.Economy.MaintenanceConsumptionPerSecondMass or bp.MaintenanceConsumptionPerSecondMass
    local upkeepEnergy = GetYield(negEnergyRate, plusEnergyRate)
    local upkeepMass = GetYield(negMassRate, plusMassRate)
    local showUpkeep = false
    if upkeepEnergy != 0 or upkeepMass != 0 then
        View.UpkeepGroup.Label:SetText(LOC("<LOC uvd_0002>Yield"))
        View.UpkeepGroup.EnergyValue:SetText( string.format("%d",upkeepEnergy) )
        View.UpkeepGroup.MassValue:SetText( string.format("%d",upkeepMass) )
        if upkeepEnergy >= 0 then
            View.UpkeepGroup.EnergyValue:SetColor( "FF50F050" )
        else
            View.UpkeepGroup.EnergyValue:SetColor( "FFF05050" )
        end
    
        if upkeepMass >= 0 then
            View.UpkeepGroup.MassValue:SetColor( "FF50F050" )
        else
            View.UpkeepGroup.MassValue:SetColor( "FFF05050" )
        end
        showUpkeep = true
    elseif bp.Economy and (bp.Economy.StorageEnergy != 0 or bp.Economy.StorageMass != 0) then
        View.UpkeepGroup.Label:SetText(LOC("<LOC uvd_0006>Storage"))
        local upkeepEnergy = bp.Economy.StorageEnergy or 0
        local upkeepMass = bp.Economy.StorageMass or 0
        View.UpkeepGroup.EnergyValue:SetText( string.format("%d",upkeepEnergy) )
        View.UpkeepGroup.MassValue:SetText( string.format("%d",upkeepMass) )
        View.UpkeepGroup.EnergyValue:SetColor( "FF50F050" )
        View.UpkeepGroup.MassValue:SetColor( "FF50F050" )
        showUpkeep = true
    end
    
    return showUpkeep 
end

function GetYield(consumption, production)
    if consumption then
        return -consumption
    elseif production then
        return production
    else
        return 0
    end
end

function Hide()
    View:Hide()
end

function SetLayout()
    import(UIUtil.GetLayoutFilename('unitviewDetail')).SetLayout()
end

function SetupUnitViewLayout(parent)
    if View then
        View:Destroy()
        View = nil
    end
    MapView = parent
    SetLayout()
    View:Hide()
end
