#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/weapon.lua
#**  Author(s):  John Comes
#**
#**  Summary  : The base weapon class for all weapons in the game.
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local Entity = import('entity.lua').Entity

Weapon = Class(moho.weapon_methods) {
    __init = function(self, unit)
        self.unit = unit
    end,

    OnCreate = function(self)
        if not self.unit.Trash then
            self.unit.Trash = TrashBag()
        end
        self:SetValidTargetsForCurrentLayer(self.unit:GetCurrentLayer())
        if self:GetBlueprint().Turreted == true then
            self:SetupTurret()
        end
        self:SetWeaponPriorities()
        self.Disabledbf = {}
        self.DamageMod = 0
        self.DamageRadiusMod = 0
        local bp = self:GetBlueprint()
        local initStore = bp.InitialProjectileStorage
        if initStore and initStore > 0 then
            if bp.MaxProjectileStorage and bp.MaxProjectileStorage < initStore then
                initStore = bp.MaxProjectileStorage
            end
            local nuke = false
            if bp.NukeWeapon then
                nuke = true
            end
            self:ForkThread(self.AmmoThread, nuke, bp.InitialProjectileStorage)
        end
    end,

    AmmoThread = function(self, nuke, amount)
        WaitSeconds(0.1)
        if nuke then
            self.unit:GiveNukeSiloAmmo(amount)
        else
            self.unit:GiveTacticalSiloAmmo(amount)
        end
    end,

    SetupTurret = function(self)
        local bp = self:GetBlueprint()
        local yawBone = bp.TurretBoneYaw
        local pitchBone = bp.TurretBonePitch
        local muzzleBone = bp.TurretBoneMuzzle
        local precedence = bp.AimControlPrecedence or 10
        local pitchBone2
        local muzzleBone2
        if bp.TurretBoneDualPitch and bp.TurretBoneDualPitch != '' then
            pitchBone2 = bp.TurretBoneDualPitch
        end
        if bp.TurretBoneDualMuzzle and bp.TurretBoneDualMuzzle != '' then
            muzzleBone2 = bp.TurretBoneDualMuzzle
        end
        if not (self.unit:ValidateBone(yawBone) and self.unit:ValidateBone(pitchBone) and self.unit:ValidateBone(muzzleBone)) then
            error('*ERROR: Bone aborting turret setup due to bone issues.', 2)
            return
        elseif pitchBone2 and muzzleBone2 then
            if not (self.unit:ValidateBone(pitchBone2) and self.unit:ValidateBone(muzzleBone2)) then
                error('*ERROR: Bone aborting turret setup due to pitch/muzzle bone2 issues.', 2)
                return
            end
        end
        if yawBone and pitchBone and muzzleBone then
            if bp.TurretDualManipulators then
                self.AimControl = CreateAimController(self, 'Torso', yawBone)
                self.AimRight = CreateAimController(self, 'Right', pitchBone, pitchBone, muzzleBone)
                self.AimLeft = CreateAimController(self, 'Left', pitchBone2, pitchBone2, muzzleBone2)
                self.AimControl:SetPrecedence(precedence)
                self.AimRight:SetPrecedence(precedence)
                self.AimLeft:SetPrecedence(precedence)
                if EntityCategoryContains(categories.STRUCTURE, self.unit) then
                    self.AimControl:SetResetPoseTime(9999999)
                end
                self:SetFireControl('Right')
                self.unit.Trash:Add(self.AimControl)
                self.unit.Trash:Add(self.AimRight)
                self.unit.Trash:Add(self.AimLeft)
            else
                self.AimControl = CreateAimController(self, 'Default', yawBone, pitchBone, muzzleBone)
                if EntityCategoryContains(categories.STRUCTURE, self.unit) then
                    self.AimControl:SetResetPoseTime(9999999)
                end
                self.unit.Trash:Add(self.AimControl)
                self.AimControl:SetPrecedence(precedence)
                if bp.RackSlavedToTurret and table.getn(bp.RackBones) > 0 then
                    for k, v in bp.RackBones do
                        if v.RackBone != pitchBone then
                            local slaver = CreateSlaver(self.unit, v.RackBone, pitchBone)
                            slaver:SetPrecedence(precedence-1)
                            self.unit.Trash:Add(slaver)
                        end
                    end
                end
            end
        else
            error('*ERROR: Trying to setup a turreted weapon but there are yaw bones, pitch bones or muzzle bones missing from the blueprint.', 2)
        end


        local numbersexist = true
        local turretyawmin, turretyawmax, turretyawspeed
        local turretpitchmin, turretpitchmax, turretpitchspeed

        #SETUP MANIPULATORS AND SET TURRET YAW, PITCH AND SPEED
        if self:GetBlueprint().TurretYaw and self:GetBlueprint().TurretYawRange then
            turretyawmin, turretyawmax = self:GetTurretYawMinMax()
        else
            numbersexist = false
        end
        if self:GetBlueprint().TurretYawSpeed then
            turretyawspeed = self:GetTurretYawSpeed()
        else
            numbersexist = false
        end
        if self:GetBlueprint().TurretPitch and self:GetBlueprint().TurretPitchRange then
            turretpitchmin, turretpitchmax = self:GetTurretPitchMinMax()
        else
            numbersexist = false
        end
        if self:GetBlueprint().TurretPitchSpeed then
            turretpitchspeed = self:GetTurretPitchSpeed()
        else
            numbersexist = false
        end
        if numbersexist then
            self.AimControl:SetFiringArc(turretyawmin, turretyawmax, turretyawspeed, turretpitchmin, turretpitchmax, turretpitchspeed)
            if self.AimRight then
                self.AimRight:SetFiringArc(turretyawmin/12, turretyawmax/12, turretyawspeed, turretpitchmin, turretpitchmax, turretpitchspeed)
            end
            if self.AimLeft then
                self.AimLeft:SetFiringArc(turretyawmin/12, turretyawmax/12, turretyawspeed, turretpitchmin, turretpitchmax, turretpitchspeed)
            end
        else
            local strg = '*ERROR: TRYING TO SETUP A TURRET WITHOUT ALL TURRET NUMBERS IN BLUEPRINT, ABORTING TURRET SETUP. WEAPON: ' .. self:GetBlueprint().Label .. ' UNIT: '.. self.unit:GetUnitId()
            error(strg, 2)
        end
    end,

    AimManipulatorSetEnabled = function(self, enabled)
        if self.AimControl then
            self.AimControl:SetEnabled(enabled)
        end
    end,

    GetAimManipulator = function(self)
        return self.AimControl
    end,

    GetAOE = function (self)
    	return self.DamageRadius
    end,
    
    SetAOE = function (self,AOE)
    	self.DamageRadius=AOE
    end,
    
    SetTurretYawSpeed = function(self, speed)
        local turretyawmin, turretyawmax = self:GetTurretYawMinMax()
        local turretpitchmin, turretpitchmax = self:GetTurretPitchMinMax()
        local turretpitchspeed = self:GetTurretPitchSpeed()
        if self.AimControl then
            self.AimControl:SetFiringArc(turretyawmin, turretyawmax, speed, turretpitchmin, turretpitchmax, turretpitchspeed)
        end
    end,

    SetTurretPitchSpeed = function(self, speed)
        local turretyawmin, turretyawmax = self:GetTurretYawMinMax()
        local turretpitchmin, turretpitchmax = self:GetTurretPitchMinMax()
        local turretpitchspeed = self:GetTurretYawSpeed()
        if self.AimControl then
            self.AimControl:SetFiringArc(turretyawmin, turretyawmax, turretyawspeed, turretpitchmin, turretpitchmax, speed)
        end
    end,

    GetTurretYawMinMax = function(self)
        local halfrange = self:GetBlueprint().TurretYawRange
        local yaw = self:GetBlueprint().TurretYaw
        turretyawmin = yaw - halfrange
        turretyawmax = yaw + halfrange
        return turretyawmin, turretyawmax
    end,

    
    ModTurretYawSpeed = function(self, YawSpeedMod)
        if not self.TurretYawSpeedMod then self.TurretYawSpeedMod = self:GetTurretYawSpeed() end
    	self.TurretYawSpeedMod = self.TurretYawSpeedMod + YawSpeedMod
    	self:SetTurretYawSpeed(self.TurretYawSpeedMod)
    	--LOG("*DEBUG: Yawspeed = ",self.TurretYawSpeedMod)
    end,
    
    GetTurretYawSpeed = function(self)
        return self:GetBlueprint().TurretYawSpeed
    end,

    GetTurretPitchMinMax = function(self)
        local halfrange = self:GetBlueprint().TurretPitchRange
        local pitch = self:GetBlueprint().TurretPitch
        turretpitchmin = pitch - halfrange
        turretpitchmax = pitch + halfrange
        return turretpitchmin, turretpitchmax
    end,

    GetTurretPitchSpeed = function(self)
        return self:GetBlueprint().TurretPitchSpeed
    end,

    OnFire = function(self)
    local bp = self:GetBlueprint()
        if bp.Audio.Fire then
            self:PlaySound(bp.Audio.Fire)
        end
        self:DoOnFireBuffs()
    end,

    OnEnableWeapon = function(self)
    end,

    OnGotTarget = function(self)
        #LOG('Got the target')
        if self.DisabledFiringBones and self.unit.Animator then
            for key, value in self.DisabledFiringBones do
                self.unit.Animator:SetBoneEnabled(value, false)
            end
        end
    end,

    OnLostTarget = function(self)
        if self.DisabledFiringBones and self.unit.Animator then
            for key, value in self.DisabledFiringBones do
                self.unit.Animator:SetBoneEnabled(value, true)
            end
        end
    end,

    OnStartTracking = function(self, label)
        self:PlayWeaponSound('BarrelStart')
        self:PlayWeaponAmbientSound('BarrelLoop')
    end,

    OnStopTracking = function(self, label)
        self:PlayWeaponSound('BarrelStop')
        self:StopWeaponAmbientSound('BarrelLoop')
        if EntityCategoryContains(categories.STRUCTURE, self.unit) then
            self.AimControl:SetResetPoseTime(9999999)
        end

    end,

    PlayWeaponSound = function(self, sound)
        local bp = self:GetBlueprint()
        if not bp.Audio[sound] then return end
        self:PlaySound(bp.Audio[sound])
    end,
    
    PlayWeaponAmbientSound = function(self, sound)
        local bp = self:GetBlueprint()
        if not bp.Audio[sound] then return end
        if not self.AmbientSounds then
            self.AmbientSounds = {}
        end
        if not self.AmbientSounds[sound] then
            local sndEnt = Entity {}
            self.AmbientSounds[sound] = sndEnt
            self.unit.Trash:Add(sndEnt)
            sndEnt:AttachTo(self.unit,-1)
        end
        self.AmbientSounds[sound]:SetAmbientSound( bp.Audio[sound], nil )
    end,
    
    StopWeaponAmbientSound = function(self, sound)
        if not self.AmbientSounds then return end
        if not self.AmbientSounds[sound] then return end
        local bp = self:GetBlueprint()
        if not bp.Audio[sound] then return end
        self.AmbientSounds[sound]:Destroy()
        self.AmbientSounds[sound] = nil
    end,

    OnEnableWeapon = function(self)
        
    end,

    OnMotionHorzEventChange = function(self, new, old)

    end,

    GetDamageTable = function(self)
        local weaponBlueprint = self:GetBlueprint()
        local damageTable = {}
        damageTable.DamageRadius = weaponBlueprint.DamageRadius + (self.DamageRadiusMod or 0)
        damageTable.DamageAmount = weaponBlueprint.Damage + (self.DamageMod or 0)
        damageTable.DamageType = weaponBlueprint.DamageType
        damageTable.DamageFriendly = weaponBlueprint.DamageFriendly
        if damageTable.DamageFriendly == nil then
            damageTable.DamageFriendly = true
        end
        damageTable.CollideFriendly = weaponBlueprint.CollideFriendly or false
        damageTable.DoTTime = weaponBlueprint.DoTTime
        damageTable.DoTPulses = weaponBlueprint.DoTPulses
        damageTable.MetaImpactAmount = weaponBlueprint.MetaImpactAmount
        damageTable.MetaImpactRadius = weaponBlueprint.MetaImpactRadius
        #Add buff
        damageTable.Buffs = {}
        if weaponBlueprint.Buffs != nil then
            for k, v in weaponBlueprint.Buffs do
                damageTable.Buffs[k] = {}
                damageTable.Buffs[k] = v
            end   
        end     
        #remove disabled buff
        if (self.Disabledbf != nil) and (damageTable.Buffs != nil) then
            for k, v in damageTable.Buffs do
                for j, w in self.Disabledbf do
                    if v.BuffType == w then
                        #Removing buff
                        table.remove( damageTable.Buffs, k )
                    end
                end
            end  
        end  
        return damageTable
    end,

    CreateProjectileForWeapon = function(self, bone)
        local proj = self:CreateProjectile(bone)
        local damageTable = self:GetDamageTable()
        if proj and not proj:BeenDestroyed() then
            proj:PassDamageData(damageTable)
            local bp = self:GetBlueprint()

            if bp.NukeOuterRingDamage and bp.NukeOuterRingRadius and bp.NukeOuterRingTicks and bp.NukeOuterRingTotalTime and
                bp.NukeInnerRingDamage and bp.NukeInnerRingRadius and bp.NukeInnerRingTicks and bp.NukeInnerRingTotalTime then
                local data = {
                    NukeOuterRingDamage = bp.NukeOuterRingDamage or 10,
                    NukeOuterRingRadius = bp.NukeOuterRingRadius or 40,
                    NukeOuterRingTicks = bp.NukeOuterRingTicks or 20,
                    NukeOuterRingTotalTime = bp.NukeOuterRingTotalTime or 10,
        
                    NukeInnerRingDamage = bp.NukeInnerRingDamage or 2000,
                    NukeInnerRingRadius = bp.NukeInnerRingRadius or 30,
                    NukeInnerRingTicks = bp.NukeInnerRingTicks or 24,
                    NukeInnerRingTotalTime = bp.NukeInnerRingTotalTime or 24,
                }
                proj:PassData(data)
            end
        end
        return proj
    end,

    SetValidTargetsForCurrentLayer = function(self, newLayer)
        #LOG( 'SetValidTargetsForCurrentLayer, layer = ', newLayer )
        local weaponBlueprint = self:GetBlueprint()
        if weaponBlueprint.FireTargetLayerCapsTable then
            if weaponBlueprint.FireTargetLayerCapsTable[newLayer] then
                #LOG( 'Setting Target Layer Caps to ', weaponBlueprint.FireTargetLayerCapsTable[newLayer] )
                self:SetFireTargetLayerCaps( weaponBlueprint.FireTargetLayerCapsTable[newLayer] )
            else
                #LOG( 'Setting Target Layer Caps to None' )
                self:SetFireTargetLayerCaps('None')
            end
        end
    end,

    OnDestroy = function(self)
    end,

    SetWeaponPriorities = function(self, priTable)
        if not priTable then
            local bp = self:GetBlueprint().TargetPriorities
            if bp then
                local priorityTable = {}
                for k, v in bp do
                    table.insert(priorityTable, ParseEntityCategory(v))
                end
                self:SetTargetingPriorities(priorityTable)
            end
        else
            self:SetTargetingPriorities(priTable)
        end
    end,

    WeaponUsesEnergy = function(self)
        local bp = self:GetBlueprint()
        if bp.EnergyRequired and bp.EnergyRequired > 0 then
            return true
        end
        return false
    end,

    ForkThread = function(self, fn, ...)
        if fn then
            local thread = ForkThread(fn, self, unpack(arg))
            self.unit.Trash:Add(thread)
            return thread
        else
            return nil
        end
    end,

    OnVeteranLevel = function(self, old, new)
        local bp = self:GetBlueprint().Buffs
        if not bp then return end

        local lvlkey = 'VeteranLevel' .. new
        for k, v in bp do
            if v.Add[lvlkey] == true then
                self:AddBuff(v)
            end
        end
    end,

    AddBuff = function(self, buffTbl)
        self.unit:AddWeaponBuff(buffTbl, self)
    end,

    AddDamageMod = function(self, dmgMod)
        self.DamageMod = self.DamageMod + dmgMod
    end,
    
    AddDamageRadiusMod = function(self, dmgRadMod)
        self.DamageRadiusMod = self.DamageRadiusMod + (dmgRadMod or 0)
    end,
    
    AddROFMod = function(self, ROFMod)
    	if not self.ROFMod then  self.ROFMod = self:GetBlueprint().RateOfFire end   
    		self.ROFMod = self.ROFMod + (ROFMod or 0)
    		self:ChangeRateOfFire(self.ROFMod)
    		--LOG("*DEBUG: New ROF=",self.ROFMod)
    end,
    
    
    ModMaxRange = function (self, RangeMod)
    	if not self.MaxRange then self.MaxRange = self:GetBlueprint().MaxRadius end
    	self.MaxRange= self.MaxRange + (RangeMod or 0)
    	return self.MaxRange   	
    end,
    
    DoOnFireBuffs = function(self)
        local data = self:GetBlueprint()
        if data.Buffs then
            for k, v in data.Buffs do
                if v.Add.OnFire == true then
                    self.unit:AddBuff(v)
                end
            end
        end
    end,

    DisableBuff = function(self, buffname)
        if buffname then
            for k, v in self.Disabledbf do
                if v == buffname then
                    #this buff is already in the table
                    return
                end
            end
            
            #Add to disabled list
            table.insert(self.Disabledbf, buffname)
        else
            #Error
            error('ERROR: DisableBuff in weapon.lua does not have a buffname') 
        end
    end,
    
    ReEnableBuff = function(self, buffname)
        if buffname then
            for k, v in self.Disabledbf do
                if v == buffname then
                    #Remove from disabled list
                    table.remove(self.Disabledbf, k)
                end
            end
        else
            #Error 
            error('ERROR: ReEnableBuff in weapon.lua does not have a buffname') 
        end
    end,
    
    #Method to mark weapon when parent unit gets loaded on to a transport unit
    SetOnTransport = function(self, transportstate)
        self.onTransport = transportstate
        if not transportstate then
            #send a message to tell the weapon that the unit just got dropped and needs to restart aim
            self:OnLostTarget()
        end
    end,

    #Method to retreive onTransport information. True if the parent unit has been loaded on to a transport unit
    GetOnTransport = function(self)
        return self.onTransport
    end,
    
    #This is the function to set a weapon enabled. 
    #If the weapon is enhabled by an enhancement, this will check to see if the unit has the enhancement before
    #allowing it to try to be enabled or disabled.
    SetWeaponEnabled = function(self, enable)
        if not enable then
            self:SetEnabled(enable)
            return
        end
        local bp = self:GetBlueprint().EnabledByEnhancement
        if bp then
            local id = self.unit:GetEntityId()
            if SimUnitEnhancements[id] then
                for k, v in SimUnitEnhancements[id] do
                    if v == bp then
                        self:SetEnabled(enable)
                        return
                    end
                end
            end
            #Enhancement needed but doesn't have it, don't allow weapon to be enabled.
            return
        end
        self:SetEnabled(enable)
    end,
}
